'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var t = _ref.types;

  var transformOnType = function transformOnType(transforms) {
    return function (node) {
      var transformer = transforms[node.type];
      if (transformer) {
        return transformer(node);
      }
      throw new Error(node.type + ' could not be transformed');
    };
  };

  var JSXIdentifier = function JSXIdentifier(node) {
    return t.stringLiteral(node.name);
  };

  var JSXNamespacedName = function JSXNamespacedName(node) {
    return t.stringLiteral(node.namespace.name + ':' + node.name.name);
  };

  var _JSXMemberExpression = transformOnType({
    JSXIdentifier: function JSXIdentifier(node) {
      return t.identifier(node.name);
    },
    JSXMemberExpression: function JSXMemberExpression(node) {
      return t.memberExpression(_JSXMemberExpression(node.object), _JSXMemberExpression(node.property));
    }
  });

  var JSXElementName = transformOnType({ JSXIdentifier: JSXIdentifier, JSXNamespacedName: JSXNamespacedName, JSXMemberExpression: _JSXMemberExpression });

  var JSXExpressionContainer = function JSXExpressionContainer(node) {
    return node.expression;
  };

  var JSXAttributeName = JSXElementName;

  var JSXAttributeValue = transformOnType({
    StringLiteral: function StringLiteral(node) {
      return node;
    },
    JSXExpressionContainer: JSXExpressionContainer
  });

  var JSXAttributes = function JSXAttributes(nodes, file) {
    var object = [];
    var objects = [];

    nodes.forEach(function (node) {
      switch (node.type) {
        case 'JSXAttribute':
          {
            if (!object) {
              object = [];
            }

            var attributeName = JSXAttributeName(node.name);

            var objectKey = _esutils2.default.keyword.isIdentifierNameES6(attributeName.value) ? t.identifier(attributeName.value) : attributeName;

            object.push(t.objectProperty(objectKey, JSXAttributeValue(node.value)));
            break;
          }
        case 'JSXSpreadAttribute':
          {
            if (object) {
              objects.push(t.objectExpression(object));
              object = null;
            }

            objects.push(node.argument);
            break;
          }
        default:
          throw new Error(node.type + ' cannot be used as a JSX attribute');
      }
    });

    if (object && object.length > 0) {
      objects.push(t.objectExpression(object));
    }

    if (objects.length === 0) {
      return t.objectExpression([]);
    } else if (objects.length === 1) {
      return objects[0];
    }

    return t.callExpression(file.addHelper('extends'), objects);
  };

  var JSXText = function JSXText(node) {
    return t.stringLiteral(node.value.replace(/\s+/g, ' '));
  };

  var JSXChild = function JSXChild(file) {
    return transformOnType({ JSXText: JSXText, JSXElement: _JSXElement(file), JSXExpressionContainer: JSXExpressionContainer });
  };

  var JSXChildren = function JSXChildren(file) {
    return function (nodes) {
      return t.arrayExpression(nodes.map(JSXChild(file)));
    };
  };

  var _JSXElement = function _JSXElement(file) {
    return function (node) {
      return t.objectExpression([t.objectProperty(t.identifier(nameProperty), JSXElementName(node.openingElement.name)), t.objectProperty(t.identifier(attributesProperty), JSXAttributes(node.openingElement.attributes, file)), t.objectProperty(t.identifier(childrenProperty), node.closingElement ? JSXChildren(file)(node.children) : t.nullLiteral())]);
    };
  };

  return {
    inherits: require('babel-plugin-syntax-jsx'),
    visitor: {
      JSXElement: function JSXElement(path, file) {
        path.replaceWith(_JSXElement(file)(path.node));
      }
    }
  };
};

var _esutils = require('esutils');

var _esutils2 = _interopRequireDefault(_esutils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var nameProperty = 'elementName';
var attributesProperty = 'attributes';
var childrenProperty = 'children';